@@ -50,7 +50,7 @@ pub struct CenterEffectItem {
     value2: Option<u32>,
 }
 
-#[derive(Serialize_repr, Deserialize_repr, Display)]
+#[derive(PartialEq, Serialize_repr, Deserialize_repr, Display)]
 #[repr(u8)]
 pub enum Rarity {
     N = 1,
@@ -96,8 +96,7 @@ pub struct ScoreItem {
 #[serde(rename_all = "camelCase")]
 pub struct PointMetrics {
     pub scores: Vec<ScoreItem>,
-    #[allow(dead_code)]
-    summary_time: DateTime<Utc>,
+    pub summary_time: DateTime<Utc>,
     pub count: u32,
 }
 
@@ -128,7 +127,7 @@ pub struct EventSchedule {
 
 #[derive(Deserialize)]
 pub struct Event {
-    id: u32,
+    pub id: u32,
     pub name: String,
     /// See `EVENT_TYPE_MAP` in constants for the mapping relationships.
     #[serde(rename = "type")]
@@ -139,7 +138,7 @@ pub struct Event {
 
 impl Display for Event {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        writeln!(f, "**{}**", self.name)?;
+        writeln!(f, "<b>{}</b>", self.name)?;
         writeln!(f, "Starts: {}", self.schedule.begin_date)?;
         writeln!(f, "Ends: {}", self.schedule.end_date)?;
         Ok(())
@@ -172,16 +171,24 @@ pub async fn get_event(event_id: u32) -> Result<Event> {
 }
 
 /// Get the metrics for an event by its ID.
-pub async fn get_event_borders(event_id: u32) -> Result<EventBorderView> {
-    let ret = Fetch::Url(Url::parse(&format!(
+pub async fn get_event_borders(event_id: u32) -> Result<Option<EventBorderView>> {
+    let resp = Fetch::Url(Url::parse(&format!(
         "https://api.matsurihi.me/mltd/v1/events/{}/rankings/borderPoints?prettyPrint=false",
         event_id
     ))?)
     .send()
     .await?
-    .json::<EventBorderView>()
+    .text()
     .await?;
-    Ok(ret)
+
+    // An "empty" reply:
+    // [
+    //   {}
+    // ]
+    if resp.starts_with("[") {
+        return Ok(None);
+    }
+    Ok(Some(serde_json::from_str::<EventBorderView>(&resp)?))
 }
 
 /// Get the IDs for all ongoing events.
